/* 1nn-1.c - Program implementing the 1-nearest-neighbor method. */

#include <stdio.h>
#include <stdlib.h>

/* PROGRAM TO MAKE PREDICTIONS USING THE 1-NEAREST-NEIGHBOR METHOD.

   Usage:

      1nn-1 instance

   Reads cases from train.n, where n is the instance number given as the
   argument.  This training data should be encoding numerically, except 
   perhaps for the target (of which there must be only one, the last item 
   on the line).  Also reads a file of inputs for test cases from test.n, and
   produces a file cguess.n, containing the predictions for the test cases
   found by the 1-nearest-neighbor method.
*/

#define Max_train 4096		/* Maximum number of training cases */
#define Max_inputs 100		/* Maximum number of inputs */
#define Max_chars 50		/* Max. number of characters in target items */

int main
( int argc,
  char **argv
)
{
  int instance;

  char filename[100];  
  FILE *f, *p;

  double train_inputs[Max_train+1][Max_inputs];
  char train_targets[Max_train+1][Max_chars+1];
  int n_inputs;
  int n_train;

  double test_inputs[Max_inputs];

  int closest[Max_train];
  int n_closest;

  double d, mind, t;
  int i, j, r;

  /* Look at arguments. */

  if (argc!=2 
   || (instance=atoi(argv[1])) <= 0 && strcmp(argv[1],"0")!=0)
  { fprintf(stderr, "Usage: 1nn-1 instance\n");
    exit(1);
  }

  /* Set random number seed. */

  srand(instance);

  /* Read training cases. */

  sprintf(filename,"train.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of training cases: %s\n",filename);
    exit(1);
  }

  for (i = 0; ; i++)
  { r = read_items(f,train_inputs[i],Max_inputs,train_targets[i],Max_chars);
    if (r==EOF) break;
    if (r<0 || i>0 && r!=n_inputs)
    { fprintf(stderr,"Bad data in train.%d\n",instance);
      exit(1);
    }
    if (i==0) n_inputs = r;
    if (i==Max_train)
    { fprintf(stderr,"Too many training cases in train.%d (max %d)\n",
        instance, Max_train);
      exit(1);
    }
  }

  if (i==0) 
  { fprintf(stderr,"File train.%d is empty\n",instance);
    exit(1);
  }

  n_train = i;

  fclose(f);

  /* Make predictions for test cases. */

  sprintf(filename,"test.%d",instance);
  f = fopen(filename,"r");
  if (f==NULL)
  { fprintf(stderr,"Can't open file of test cases: %s\n",filename);
    exit(1);
  }

  sprintf(filename,"cguess.%d",instance);
  p = fopen(filename,"w");
  if (p==NULL)
  { fprintf(stderr,"Can't create file of guesses: %s\n",filename);
    exit(1);
  }

  for (;;)
  { 
    r = read_items(f,test_inputs,Max_inputs,(char*)0,0);
    if (r==EOF) break;
    if (r!=n_inputs)
    { fprintf(stderr,"Bad data in test.%d\n",instance);
      exit(1);
    }

    for (i = 0; i<n_train; i++)
    { d = 0;
      for (j = 0; j<n_inputs; j++)
      { t = train_inputs[i][j] - test_inputs[j];
        d += t*t;
      }
      if (i==0 || d<=mind)
      { if (i==0 || d<mind)
        { n_closest = 1;
          closest[0] = i;
          mind = d;
        }
        else
        { closest[n_closest] = i;
          n_closest += 1;
        }
      }
    }

    i = closest[(rand()/17)%n_closest];
    fprintf(p,"%s\n",train_targets[i]);
  }

  fclose(f);
  fclose(p);

  exit(0);
}
