/* READ-ITEMS.C - Procedure for reading items from a line. */

#include <stdio.h>


/* READ ITEMS ON A LINE.  Reads the next line of input from the given file, 
   and separates out the whitespace-delimited items.  All but (perhaps) the 
   last item must be numeric; their values are stored in the 'item' array 
   passed.  If 'last' in non-zero, the last item may be anything; it's value 
   is stored in the 'last' string passed.  If 'last' is zero, the last item
   is not treated specially; all items are put in the 'item' array.

   The value returned is normally the number of items on the line, excluding 
   the special last one (of 'last' is non-zero).  EOF is returned if 
   end-of-file was encountered, and -2 is returned if the contents of the 
   line were invalid.
*/

int read_items
( FILE *f,	/* File to read from */
  double *item,	/* Place to store numeric items, needs at least max_items */
  int max_items,/* Maximum number of items in 'item' */
  char *last,	/* Place to store last item, needs at least max_chars+1 bytes */
  int max_chars /* Maximum number of characters in last item */
)
{
  int i;	/* Number of numeric items read so far */
  int j;	/* Number of characters read in current item */
  int c;	/* Current character */

  char ich[101];/* Characters making up item */
  char jch;	/* Place to put junk character */
  
  i = 0;
  ich[0] = 0;

  c = getc(f);

  for (;;)
  { 
    while (c==' ' || c=='\t') 
    { c = getc(f);
    }

    if (c==EOF)
    { if (i>0) return -2;
      return EOF;
    }

    if (ich[0]!=0)
    { if (c=='\n' && last!=0)
      { if (strlen(ich)>max_chars) return -2;
        strcpy(last,ich);
      }
      else
      { if (i==max_items) return -2;
        if (sscanf(ich,"%lf%c",&item[i],&jch)!=1) return -2;
        i += 1;
      } 
    }

    if (c=='\n')
    { if (last!=0 && ich[0]==0) return -2;
      return i;
    }

    for (j = 0; c!=' ' && c!='\t' && c!='\n' && c!=EOF; j++)
    { if (j==100) return -2;
      ich[j] = c;
      c = getc(f);
    }

    ich[j] = 0;

  }
}
